/*
Thanks to Lion Kimbro for his great prticle code. You can reach him via email
at SnowLion@sprynet.com
*/

extern int NullParticle;

void UndrawParticles();
void HandleParticles();
void DisplayParticles();
void NewParticles(float turn, int X, int Y, float spread_percent);
